classdef CsfReader < handle
  % CsfReader
  %   This class reads csf files
  %
  % Methods:
  %   readFrame - Read the next data frame from disk
  %   readFrameIntoPointer - Read the next data frame from disk into the
  %                          supplied pointer
  %   numberOfFrames - Get the total number of frames in the csf file
  %   getHeader - Returns an empty Data class containing the header information
  %               for the specified frame
  %   nextStream - Get the stream information of the next frame
  %   nextFrameSize - Get the size of the next frame
  %   frameIndex - Get the current frame position
  %   goToFrame - Go to specified frame
  %   streams - Get the streams that are in the csf file
  %   streamCount - Get the number of frames contained in the file of the given
  %                 stream
  %   goToStream - Go to the specified stream, the next read frame call will
  %                start from here
  %   getCalibration - Get calibration from the csf file
  %   getCameraConfig - Get camera config from the csf file
  %

  properties(Access=protected)
    mPtr = uint64(0);
  end

  methods(Access=public)
    function obj = CsfReader(varargin)
      % CsfReader
      %   Construct new csf_reader class
      %
      % ----------
      %
      %   Open a csf file for reading
      %
      % Inputs:
      %   1 - string - File location of the csf file
      %

      if nargin == 1 && isa(varargin{1}, 'uint64')
        obj.mPtr = varargin{1};
      elseif nargin == 1 && (isStringScalar(varargin{1}) || ischar(varargin{1}))
        obj.mPtr = tof.mex('tof_csf_reader_new', varargin{1});
      elseif nargin == 0 && (isStringScalar(varargin{1}) || ischar(varargin{1}))
        obj.mPtr = tof.mex('tof_csf_reader_new', );
      else
        error("Can't construct empty object");
      end
    end


    function delete(obj)
      if obj.mPtr ~= 0
        tof.mex('tof_csf_reader_delete', obj.mPtr);
        obj.mPtr = 0;
      end
    end

    function ptr = get_object_ptr(obj)
      ptr = obj.mPtr;
    end

    function ret_val = readFrame(obj)
      % readFrame
      %   Read the next data frame from disk
      %
      % Output:
      %   ret_val - Data - A data frame
      %

      ret_val = tof.mex('tof_csf_reader_read_frame', obj.mPtr);
      ret_val = tof.Data(ret_val);
    end

    function ret_val = numberOfFrames(obj)
      % numberOfFrames
      %   Get the total number of frames in the csf file
      %
      % Output:
      %   ret_val - numeric - Total number of frames in the csf file
      %

      ret_val = tof.mex('tof_csf_reader_number_of_frames', obj.mPtr);
      ret_val = double(ret_val);
    end

    function ret_val = getHeader(obj, index)
      % getHeader
      %   Returns an empty Data class containing the header information for the
      %   specified frame
      %
      % Inputs:
      %   index - numeric - Which frame to get the header from
      %
      % Output:
      %   ret_val - Data - Empty Data instance containing header information
      %

      ret_val = tof.mex('tof_csf_reader_get_header', obj.mPtr, index);
      ret_val = tof.Data(ret_val);
    end

    function ret_val = nextStream(obj)
      % nextStream
      %   Get the stream information of the next frame
      %
      % Output:
      %   ret_val - Stream - Information on the next frame
      %

      ret_val = tof.mex('tof_csf_reader_next_stream', obj.mPtr);
      ret_val = tof.Stream(ret_val);
    end

    function ret_val = nextFrameSize(obj)
      % nextFrameSize
      %   Get the size of the next frame
      %
      % Output:
      %   ret_val - numeric - Size of the next frame
      %

      ret_val = tof.mex('tof_csf_reader_next_frame_size', obj.mPtr);
      ret_val = double(ret_val);
    end

    function ret_val = frameIndex(obj)
      % frameIndex
      %   Get the current frame position
      %
      % Output:
      %   ret_val - numeric - Frame index
      %

      ret_val = tof.mex('tof_csf_reader_frame_index', obj.mPtr);
      ret_val = double(ret_val);
    end

    function goToFrame(obj, index)
      % goToFrame
      %   Go to specified frame
      %
      % Inputs:
      %   index - numeric - The index of the frame to jump to
      %

      tof.mex('tof_csf_reader_go_to_frame', obj.mPtr, index);
    end

    function ret_val = streams(obj)
      % streams
      %   Get the streams that are in the csf file
      %
      % Output:
      %   ret_val - vector[Stream] - Streams in the csf file
      %

      ret_val = tof.mex('tof_csf_reader_streams', obj.mPtr);
      ret_val = arrayfun(@(vec_obj)tof.Stream(vec_obj), ret_val);
    end

    function ret_val = streamCount(obj, stream)
      % streamCount
      %   Get the number of frames contained in the file of the given stream
      %
      % Inputs:
      %   stream - Stream - The stream
      %
      % Output:
      %   ret_val - numeric - Number of frames for given stream
      %

      ret_val = tof.mex('tof_csf_reader_stream_count', obj.mPtr, stream.get_object_ptr());
      ret_val = double(ret_val);
    end

    function goToStream(obj, stream, index)
      % goToStream
      %   Go to the specified stream, the next read frame call will start from
      %   here
      %
      % Inputs:
      %   stream - Stream - The stream
      %   index - numeric - The frame index
      %

      tof.mex('tof_csf_reader_go_to_stream', obj.mPtr, stream.get_object_ptr(), index);
    end

    function ret_val = getCalibration(obj)
      % getCalibration
      %   Get calibration from the csf file
      %
      % Output:
      %   ret_val - Calibration - Calibration
      %

      ret_val = tof.mex('tof_csf_reader_get_calibration', obj.mPtr);
      ret_val = tof.Calibration(ret_val);
    end

    function ret_val = getCameraConfig(obj)
      % getCameraConfig
      %   Get camera config from the csf file
      %
      % Output:
      %   ret_val - CameraConfig - Camera config
      %

      ret_val = tof.mex('tof_csf_reader_get_camera_config', obj.mPtr);
      ret_val = tof.CameraConfig(ret_val);
    end

  end
end
